@@ -177,9 +177,11 @@ ConeProperties all_options() {
     ret.set(ConeProperty::MaxDegRepresentations);
     ret.set(ConeProperty::ConeForMonoid);
     ret.set(ConeProperty::UseWeightsPatching);
+    ret.set(ConeProperty::NoWeights);
     ret.set(ConeProperty::LinearOrderPatches);
     ret.set(ConeProperty::CongOrderPatches);
     ret.set(ConeProperty::MinimizePolyEquations);
+    ret.set(ConeProperty::DistributedComp);
     return ret;
 }
 
@@ -216,6 +218,8 @@ ConeProperties treated_as_hom_props() {
     ret.set(ConeProperty::DualIncidence);
     ret.set(ConeProperty::DualFVector);
     ret.set(ConeProperty::DualFaceLattice);
+    ret.set(ConeProperty::DualFVectorOrbits);
+    ret.set(ConeProperty::DualFaceLatticeOrbits);
     return ret;
 }
 
@@ -314,6 +318,28 @@ void ConeProperties::check_lattice_ideal_goals() const{
     }
 }
 
+void ConeProperties::check_fusion_ring_props() const{
+    ConeProperties copy(*this);
+    copy.reset(ConeProperty::FusionRings);
+    copy.reset(ConeProperty::SimpleFusionRings);
+    copy.reset(ConeProperty::FusionData);
+    copy.reset(ConeProperty::LatticePoints);
+    copy.reset(ConeProperty::SingleLatticePointInternal);
+    copy.reset(ConeProperty::SingleLatticePoint);
+    copy.reset(ConeProperty::LinearOrderPatches);
+    copy.reset(ConeProperty::CongOrderPatches);
+    copy.reset(ConeProperty::UseWeightsPatching);
+    copy.reset(ConeProperty::NoWeights);
+    copy.reset(ConeProperty::NoGradingDenom);
+    copy.reset(ConeProperty::DistributedComp);
+    copy.reset(ConeProperty::Projection);
+
+    if (copy.any()) {
+        errorOutput() << copy << endl;
+        throw BadInputException("Cone Property in last line not allowed for fusion rings");
+    }
+}
+
 ConeProperties all_automorphisms() {
     static ConeProperties ret;
     ret.set(ConeProperty::Automorphisms);
@@ -380,9 +406,46 @@ size_t ConeProperties::count() const {
     return CPs.count();
 }
 
+void ConeProperties::set_fusion_default(const bool has_subring) {
+
+    if(CPs.test(ConeProperty::LatticePoints) || CPs.test(ConeProperty::FusionRings)
+        || CPs.test(ConeProperty::SimpleFusionRings) || CPs.test(ConeProperty::NonsimpleFusionRings)
+         || CPs.test(ConeProperty::FusionData) )
+        return;
+    if(CPs.test(ConeProperty::DefaultMode)){
+        if(has_subring)
+            CPs.set(ConeProperty::SimpleFusionRings);
+        else
+            CPs.set(ConeProperty::FusionRings);
+        CPs.reset(ConeProperty::DefaultMode);
+    }
+}
+
+void ConeProperties::set_fusion_partition_default() {
+
+    if(CPs.test(ConeProperty::LatticePoints))
+        return;
+    if(CPs.test(ConeProperty::DefaultMode)){
+        CPs.set(ConeProperty::SingleLatticePoint);
+        CPs.reset(ConeProperty::DefaultMode);
+    }
+}
+
 /* add preconditions */
 void ConeProperties::set_preconditions(bool inhomogeneous, bool numberfield) {
 
+    if(CPs.test(ConeProperty::NonsimpleFusionRings)){
+        CPs.set(ConeProperty::FusionRings);
+        CPs.reset(ConeProperty::NonsimpleFusionRings);
+    }
+
+    if(CPs.test(ConeProperty::FusionData) && !CPs.test(ConeProperty::SimpleFusionRings)){
+        CPs.set(ConeProperty::FusionRings);
+    }
+
+    // if(CPs.test(ConeProperty::FusionRings) || CPs.test(ConeProperty::SimpleFusionRings))
+    //    CPs.set(ConeProperty::LatticePoints); // LatticePoints will be disabled later
+
     if(CPs.test(ConeProperty::GroebnerBasis) ||
                 CPs.test(ConeProperty::MarkovBasis)){
         CPs.set(ConeProperty::HilbertBasis);
@@ -718,6 +781,10 @@ void ConeProperties::check_compatibility_with_polynomial_constraints(bool inhomo
     wanted.reset(ConeProperty::NumberLatticePoints);
     wanted.reset(ConeProperty::SingleLatticePoint);
     wanted.reset(ConeProperty::DistributedComp);
+    wanted.reset(ConeProperty::FusionRings);
+    wanted.reset(ConeProperty::SimpleFusionRings);
+    wanted.reset(ConeProperty::FusionData);
+    wanted.reset(ConeProperty::NonsimpleFusionRings);
     if(inhomogeneous)
         wanted.reset(ConeProperty::HilbertBasis);
     if(wanted.any()){
@@ -777,6 +844,10 @@ void ConeProperties::check_Q_permissible(bool after_implications) {
     copy.reset(ConeProperty::DualFaceLattice);
     copy.reset(ConeProperty::DualFVector);
     copy.reset(ConeProperty::DualIncidence);
+    copy.reset(ConeProperty::FaceLatticeOrbits);
+    copy.reset(ConeProperty::FVectorOrbits);
+    copy.reset(ConeProperty::DualFaceLatticeOrbits);
+    copy.reset(ConeProperty::DualFVectorOrbits);
     copy.reset(ConeProperty::AmbientAutomorphisms);
     copy.reset(ConeProperty::InputAutomorphisms);
     copy.reset(ConeProperty::Automorphisms);
@@ -794,7 +865,11 @@ void ConeProperties::check_Q_permissible(bool after_implications) {
     copy.reset(ConeProperty::SingleLatticePointInternal);
     copy.reset(ConeProperty::NoCoarseProjection);
     copy.reset(ConeProperty::NoPatching);
-
+    copy.reset(ConeProperty::FusionRings);
+    copy.reset(ConeProperty::SimpleFusionRings);
+    copy.reset(ConeProperty::NonsimpleFusionRings);
+    copy.reset(ConeProperty::FusionData);
+;
     if (after_implications) {
         copy.reset(ConeProperty::Multiplicity);
         copy.reset(ConeProperty::Grading);
@@ -818,8 +893,13 @@ void ConeProperties::check_conflicting_variants() {
         (CPs.test(ConeProperty::NoSignedDec) && CPs.test(ConeProperty::SignedDec)) ||
         (CPs.test(ConeProperty::Symmetrize) && CPs.test(ConeProperty::Descent)) ||
         (CPs.test(ConeProperty::Descent) && CPs.test(ConeProperty::SignedDec)) ||
+        (CPs.test(ConeProperty::UseWeightsPatching) && CPs.test(ConeProperty::NoWeights)) ||
+        (CPs.test(ConeProperty::LinearOrderPatches) && CPs.test(ConeProperty::CongOrderPatches)) ||
+        (CPs.test(ConeProperty::LatticePoints) && CPs.test(ConeProperty::SingleLatticePoint) ) ||
         // (CPs.test(ConeProperty::Symmetrize) && CPs.test(ConeProperty::SignedDec)) ||
-        (CPs.test(ConeProperty::Dynamic) && CPs.test(ConeProperty::Static)))
+        (CPs.test(ConeProperty::Dynamic) && CPs.test(ConeProperty::Static)) ||
+        (CPs.test(ConeProperty::FusionRings) && CPs.test(ConeProperty::SimpleFusionRings))
+    )
         throw BadInputException("Contradictory algorithmic variants in options.");
 
     size_t nr_var = 0;
@@ -874,12 +954,24 @@ void ConeProperties::check_sanity(bool inhomogeneous) {  //, bool input_automorp
 
     bool something_to_do_primal =
         CPs.test(ConeProperty::FaceLattice) || CPs.test(ConeProperty::FVector) || CPs.test(ConeProperty::Incidence);
+    bool something_to_do_primal_orbits = test(ConeProperty::FaceLatticeOrbits) || CPs.test(ConeProperty::FVectorOrbits);
 
     bool something_to_do_dual =
         CPs.test(ConeProperty::DualFaceLattice) || CPs.test(ConeProperty::DualFVector) || CPs.test(ConeProperty::DualIncidence);
+    bool something_to_do_dual_orbits =test(ConeProperty::DualFaceLatticeOrbits) || CPs.test(ConeProperty::DualFVectorOrbits);
 
-    if (something_to_do_dual && something_to_do_primal)
-        throw BadInputException("Only one of primal or dual face lattice/f-vector/incidence allowed");
+    int nr_face_kattice_goals = 0;
+    if(something_to_do_primal)
+        nr_face_kattice_goals++;
+    if(something_to_do_primal_orbits)
+        nr_face_kattice_goals++;
+    if(something_to_do_dual)
+        nr_face_kattice_goals++;
+    if(something_to_do_dual_orbits)
+        nr_face_kattice_goals++;
+
+    if (nr_face_kattice_goals > 1)
+        throw BadInputException("Only one of primal/dual full/orbits face lattice/f-vector/incidence allowed");
 
     if (intersection_with(all_automorphisms()).count() > 1)
         throw BadInputException("Only one type of automorphism group allowed.");
@@ -1023,9 +1115,13 @@ vector<string> initializeCPN() {
     CPN.at(ConeProperty::NumberLatticePoints) = "NumberLatticePoints";
     CPN.at(ConeProperty::FaceLattice) = "FaceLattice";
     CPN.at(ConeProperty::FVector) = "FVector";
-    CPN.at(ConeProperty::Incidence) = "Incidence";
     CPN.at(ConeProperty::DualFaceLattice) = "DualFaceLattice";
     CPN.at(ConeProperty::DualFVector) = "DualFVector";
+    CPN.at(ConeProperty::FaceLatticeOrbits) = "FaceLatticeOrbits";
+    CPN.at(ConeProperty::FVectorOrbits) = "FVectorOrbits";
+    CPN.at(ConeProperty::DualFaceLatticeOrbits) = "DualFaceLatticeOrbits";
+    CPN.at(ConeProperty::DualFVectorOrbits) = "DualFVectorOrbits";
+    CPN.at(ConeProperty::Incidence) = "Incidence";
     CPN.at(ConeProperty::DualIncidence) = "DualIncidence";
     CPN.at(ConeProperty::SingularLocus) = "SingularLocus";
     CPN.at(ConeProperty::CodimSingularLocus) = "CodimSingularLocus";
@@ -1048,12 +1144,17 @@ vector<string> initializeCPN() {
     CPN.at(ConeProperty::MaxDegRepresentations) = "MaxDegRepresentations";
     CPN.at(ConeProperty::ConeForMonoid) = "ConeForMonoid";
     CPN.at(ConeProperty::UseWeightsPatching) = "UseWeightsPatching";
+    CPN.at(ConeProperty::NoWeights) = "NoWeights";
     CPN.at(ConeProperty::MinimizePolyEquations) = "MinimizePolyEquations";
     CPN.at(ConeProperty::LinearOrderPatches) = "LinearOrderPatches";
     CPN.at(ConeProperty::CongOrderPatches) = "CongOrderPatches";
+    CPN.at(ConeProperty::FusionRings) = "FusionRings";
+    CPN.at(ConeProperty::SimpleFusionRings) = "SimpleFusionRings";
+    CPN.at(ConeProperty::NonsimpleFusionRings) = "NonsimpleFusionRings";
+    CPN.at(ConeProperty::FusionData) = "FusionData";
 
     // detect changes in size of Enum, to remember to update CPN!
-    static_assert(ConeProperty::EnumSize == 151, "ConeProperties Enum size does not fit! Update cone_property.cpp!");
+    static_assert(ConeProperty::EnumSize == 160, "ConeProperties Enum size does not fit! Update cone_property.cpp!");
     // assert all fields contain an non-empty string
     for (size_t i = 0; i < ConeProperty::EnumSize; i++) {
         // bstd::cout << "iii " << i << "  " << CPN.at(i) << endl;
