@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -25,10 +25,152 @@
 #include <cctype>  // std::isdigit
 #include <limits>  // numeric_limits
 
+// #include <boost/smart_ptr/intrusive_ptr.hpp>
+
 #include "libnormaliz/input.h"
+#include "libnormaliz/list_and_map_operations.h"
 
 namespace libnormaliz {
 
+//---------------------------------------------------------------------------
+//                     Number input
+//---------------------------------------------------------------------------
+
+
+#ifdef ENFNORMALIZ
+
+static int xalloc = std::ios_base::xalloc();
+
+// Normaliz implementation of deprecated e-antic functions
+std::istream & nmz_set_pword(boost::intrusive_ptr<const renf_class> our_renf, std::istream & is)
+{
+    is.pword(xalloc) = const_cast<void*>(reinterpret_cast<const void*>(&*our_renf));
+    return is;
+}
+boost::intrusive_ptr<const renf_class> nmz_get_pword(std::istream& is) {
+    return reinterpret_cast<renf_class*>(is.pword(xalloc));
+}
+#endif
+//---------------------------------------------------------
+
+
+// To be used in input.cpp
+inline void string2coeff(mpq_class& coeff, istream& in, const string& s) {  // in here superfluous parameter
+
+    stringstream sin(s);
+    coeff = mpq_read(sin);
+    // coeff=mpq_class(s);
+}
+
+inline void read_number(istream& in, mpq_class& number) {
+    number = mpq_read(in);
+}
+
+inline void read_number(istream& in, long& number) {
+    in >> number;
+}
+
+inline void read_number(istream& in, long long& number) {
+    in >> number;
+}
+
+inline void read_number(istream& in, nmz_float& number) {
+    in >> number;
+}
+
+inline void read_number(istream& in, mpz_class& number) {
+    in >> number;
+}
+
+#ifdef ENFNORMALIZ
+
+inline void string2coeff(renf_elem_class& coeff, istream& in, const string& s) {  // we need in to access the renf
+
+    try {
+        coeff = renf_elem_class(*nmz_get_pword(in), s);
+    } catch (const std::exception& e) {
+        cerr << e.what() << endl;
+        throw BadInputException("Illegal number string " + s + " in input, Exiting.");
+    }
+}
+
+inline void read_number(istream& in, renf_elem_class& number) {
+    // in >> number;
+
+    char c;
+
+    in >> ws;
+    c = in.peek();
+    if (c != '(' && c != '\'' && c != '\"') {  // rational number
+        mpq_class rat = mpq_read(in);
+        number = renf_elem_class(rat);
+        return;
+    }
+
+    // now we have a proper field element
+
+    in >> c;  // read (
+
+    string num_string;
+    bool skip = false;
+    while (in.good()) {
+        c = in.peek();
+        if (c == ')' || c == '\'' || c == '\"') {
+            in >> c;
+            break;
+        }
+        if (c == '~' || c == '=' || c == '[')  // skip the approximation
+            skip = true;
+        in.get(c);
+        if (in.fail())
+            throw BadInputException("Error in reading number: field element not terminated");
+        if (!skip)
+            num_string += c;
+    }
+    string2coeff(number, in, num_string);
+}
+#endif
+
+// matrix input
+
+template <typename Integer>
+Matrix<Integer> readMatrix(const string project) {
+    // reads one matrix from file with name project
+    // format: nr of rows, nr of colimns, entries
+    // all separated by white space
+
+    string name_in = project;
+    const char* file_in = name_in.c_str();
+    ifstream in;
+    in.open(file_in, ifstream::in);
+    if (in.is_open() == false)
+        throw BadInputException("readMatrix cannot find file " + project);
+    int nrows, ncols;
+    in >> nrows;
+    in >> ncols;
+
+    Matrix<Integer> result(nrows, ncols);
+
+    if (nrows == 0 || ncols == 0){
+        if(verbose){
+            verboseOutput() << "Matrix in file " << project << " empty" << endl;
+        }
+        return result;
+    }
+
+    int i, j;
+
+    for (i = 0; i < nrows; ++i)
+        for (j = 0; j < ncols; ++j) {
+            read_number(in, result[i][j]);
+            if (in.fail())
+                throw BadInputException("readMatrix finds matrix corrupted");
+        }
+    return result;
+}
+
+//--------------------------------------------------------------------------
+
 // eats up a comment, stream must start with "/*", eats everything until "*/"
 void skip_comment(istream& in) {
     int i = in.get();
@@ -46,41 +188,42 @@ void skip_comment(istream& in) {
 }
 
 template <typename Number>
-void save_matrix(map<Type::InputType, vector<vector<Number> > >& input_map,
+void save_matrix(map<Type::InputType, Matrix<Number> >& input_map,
                  InputType input_type,
-                 const vector<vector<Number> >& M) {
+                 const Matrix<Number>& M) {
     // check if this type already exists
     if (contains(input_map, input_type)) {
         /*throw BadInputException("Multiple inputs of type \"" + type_string
                 + "\" are not allowed!");*/
-        input_map[input_type].insert(input_map[input_type].end(), M.begin(), M.end());
+        input_map[input_type].append(M);
         return;
     }
     input_map[input_type] = M;
 }
 
 template <typename Number>
-void save_empty_matrix(map<Type::InputType, vector<vector<Number> > >& input_map, InputType input_type) {
-    vector<vector<Number> > M;
+void save_empty_matrix(map<Type::InputType, Matrix<Number> >& input_map, InputType input_type) {
+    Matrix<Number> M;
     save_matrix(input_map, input_type, M);
 }
-
+ /*
 template <typename Number>
-vector<vector<Number> > transpose_mat(const vector<vector<Number> >& mat) {
-    if (mat.size() == 0 || mat[0].size() == 0)
-        return vector<vector<Number> >(0);
+Matrix<Number> transpose_mat(const Matrix<Number>& mat) {
+    if (mat.nr_of_rows() == 0 || mat[0].size() == 0)
+        return Matrix<Number>(0);
     size_t m = mat[0].size();
-    size_t n = mat.size();
-    vector<vector<Number> > transpose(m, vector<Number>(n, 0));
+    size_t n = mat.nr_of_rows()
+    Matrix<Number> transpose(m, vector<Number>(n, 0));
     for (size_t i = 0; i < m; ++i)
         for (size_t j = 0; j < n; ++j)
             transpose[i][j] = mat[j][i];
     return transpose;
 }
+*/
 
 template <typename Number>
-void append_row(const vector<Number> row, map<Type::InputType, vector<vector<Number> > >& input_map, Type::InputType input_type) {
-    vector<vector<Number> > one_row(1, row);
+void append_row(const vector<Number> row, map<Type::InputType, Matrix<Number> >& input_map, Type::InputType input_type) {
+    Matrix<Number> one_row(row);
     save_matrix(input_map, input_type, one_row);
 }
 
@@ -89,7 +232,7 @@ void process_constraint(const string& rel,
                         const vector<Number>& left,
                         Number right,
                         const Number modulus,
-                        map<Type::InputType, vector<vector<Number> > >& input_map,
+                        map<Type::InputType, Matrix<Number> >& input_map,
                         bool forced_hom) {
     vector<Number> row = left;
     bool inhomogeneous = false;
@@ -110,6 +253,9 @@ void process_constraint(const string& rel,
     if (strict_inequality && forced_hom) {
         throw BadInputException("Strict inequality not allowed in hom_constraints!");
     }
+    if (strict_inequality && using_renf<Number>()) {
+        throw BadInputException("Strict inequality not allowed for algebraic polyhedra!");
+    }
     if (inhomogeneous || forced_hom)
         row.push_back(-right);   // rhs --> lhs
     if (modified_rel == "<=") {  // convert <= to >=
@@ -117,8 +263,11 @@ void process_constraint(const string& rel,
             row[j] = -row[j];
         modified_rel = ">=";
     }
-    if (rel == "~")
+    if (rel == "~") {
+        if (using_renf<Number>())
+            throw BadInputException("Congruence not allowed for algebraic polyhedra");
         row.push_back(modulus);
+    }
 
     if (inhomogeneous && !forced_hom) {
         if (modified_rel == "=") {
@@ -148,11 +297,13 @@ void process_constraint(const string& rel,
             return;
         }
     }
-    throw BadInputException("Illegal constrint type " + rel + " !");
+    throw BadInputException("Illegal constraint type " + rel + " !");
 }
 
 template <typename Number>
 bool read_modulus(istream& in, Number& modulus) {
+    if (using_renf<Number>())
+        throw BadInputException("Congruence not allowed for field coefficients");
     in >> std::ws;  // gobble any leading white space
     char dummy;
     in >> dummy;
@@ -168,6 +319,17 @@ bool read_modulus(istream& in, Number& modulus) {
     return true;
 }
 
+void check_modulus(mpq_class& modulus) {
+    if (modulus <= 0 || modulus.get_den() != 1)
+        throw BadInputException("Error in modulus of congruence");
+}
+
+#ifdef ENFNORMALIZ
+void check_modulus(renf_elem_class& modulus) {
+    throw BadInputException("Congruences not allowed for algebraic polyhedra");
+}
+#endif
+
 template <typename Number>
 void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Number& right, Number& modulus, bool forced_hom) {
     string constraint;
@@ -279,6 +441,7 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
         current_term += test;
     }
     terms.push_back(current_term);
+    // cout << terms;
     if (!relation_read)
         throw BadInputException("No relation in constraint");
 
@@ -287,6 +450,8 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
 
     // now we split off the modulus if necessary
     if (rel == "~") {
+        if (using_renf<Number>())
+            throw BadInputException("Congruence not allowed for algebraic polyhedra");
         string last_term = terms.back();
         size_t last_bracket_at = 0;
         bool has_bracket = false;
@@ -305,8 +470,7 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
         modulus = mpq_class(modulus_string);
         // modulus.canonicalize();
         // cout << "mod " << modulus << endl;
-        if (modulus <= 0 || modulus.get_den() != 1)
-            throw BadInputException("Error in modulus of congruence");
+        check_modulus(modulus);
     }
 
     // for(size_t i=0;i<terms.size();++i)
@@ -352,7 +516,7 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
         if (coeff_length == 0 || (coeff_length == 1 && coeff_string[0] == '+'))
             coeff = 1;
         if (coeff_length == 1 && coeff_string[0] == '-')
-            coeff = -11;
+            coeff = -1;
         if (coeff == 0) {
             // cout << i << " coeff string: " << coeff_string << endl;
             const string numeric = "+-0123456789/a^*().e";
@@ -370,12 +534,16 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
         }
 
         if (comp_string != "") {
+
+            // cout << "CCCCCCCCCC " << comp_string << endl;
             bool bracket_read = false;
             string expo_string;
             for (char j : comp_string) {
                 if (j == ']')
                     break;
                 if (j == '[') {
+                    if(bracket_read)
+                        throw BadInputException("Double [ in term.");
                     bracket_read = true;
                     continue;
                 }
@@ -400,7 +568,7 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
 }
 
 template <typename Number>
-void read_constraints(istream& in, long dim, map<Type::InputType, vector<vector<Number> > >& input_map, bool forced_hom) {
+void read_constraints(istream& in, long dim, map<Type::InputType, Matrix<Number> >& input_map, bool forced_hom) {
     long nr_constraints;
     in >> nr_constraints;
 
@@ -443,7 +611,6 @@ void read_constraints(istream& in, long dim, map<Type::InputType, vector<vector<
             read_number(in, right);
             if (rel == "~") {
                 if (!read_modulus(in, modulus))
-                    // throw BadInputException("Congruence not allowed with field coefficients!");
                     throw BadInputException("Error while reading modulus!");
             }
             if (in.fail()) {
@@ -461,28 +628,48 @@ bool read_sparse_vector(istream& in, vector<Number>& input_vec, long length) {
 
     while (in.good()) {
         in >> std::ws;
-        int c = in.peek();
+        char c = in.peek();
         if (c == ';') {
             in >> dummy;  // swallow ;
             return true;
         }
-        long pos;
-        in >> pos;
-        if (in.fail())
+        string range;
+        while (true) {
+            in >> c;
+            if (in.fail())
+                return false;
+            if (c != ':')
+                range += c;
+            else
+                break;
+        }
+        int first_pos = -1, last_pos = -1;
+        size_t found_dots = range.find("..", 0);
+        if (found_dots != string::npos) {
+            if (found_dots == 0)
+                return false;
+            first_pos = stoi(range.substr(0, found_dots));
+            first_pos--;
+            last_pos = stoi(range.substr(found_dots + 2));
+            last_pos--;
+        }
+        else {
+            first_pos = stoi(range);
+            first_pos--;
+            last_pos = first_pos;
+        }
+
+        if (first_pos < 0 || first_pos >= length)
             return false;
-        pos--;
-        if (pos < 0 || pos >= length)
+        if (last_pos < first_pos || last_pos >= length)
             return false;
-        in >> std::ws;
-        c = in.peek();
-        if (c != ':')
-            return false;
-        in >> dummy;  // skip :
+
         Number value;
         read_number(in, value);
         if (in.fail())
             return false;
-        input_vec[pos] = value;
+        for (int i = first_pos; i <= last_pos; ++i)
+            input_vec[i] = value;
     }
 
     return false;
@@ -533,8 +720,26 @@ void read_polynomial(istream& in, string& polynomial) {
     }
 }
 
+void read_polynomial_constraints(istream& in, vector<string>& polynomial_constraints) {
+
+    int nr_constraints;
+    in >> nr_constraints;
+    if(in.fail() || nr_constraints< 0)
+        throw BadInputException("Failure in reading number of polynomial constraints!");
+    if(nr_constraints == 0)
+        return;
+
+    string equ;
+    for(int i = 0; i < nr_constraints; ++i){
+        read_polynomial(in, equ);
+        polynomial_constraints.push_back(equ);
+        equ.clear();
+    }
+}
+
 template <typename Number>
-bool read_formatted_matrix(istream& in, vector<vector<Number> >& input_mat, bool transpose) {
+bool read_formatted_matrix(istream& in, Matrix<Number>& input_Matrix, bool transpose) {
+    vector<vector<Number> > input_mat;
     input_mat.clear();
     in >> std::ws;
     char dummy;
@@ -546,8 +751,9 @@ bool read_formatted_matrix(istream& in, vector<vector<Number> >& input_mat, bool
         in >> std::ws;
         if (!one_more_entry_required && in.peek() == ']') {  // closing ] found
             in >> dummy;
+            input_Matrix = Matrix<Number>(input_mat);
             if (transpose)
-                input_mat = transpose_mat(input_mat);
+                input_Matrix = input_Matrix.transpose();
             return true;
         }
         vector<Number> input_vec;
@@ -569,14 +775,7 @@ bool read_formatted_matrix(istream& in, vector<vector<Number> >& input_mat, bool
     return false;
 }
 
-template <typename Number>
-void read_number_field(istream& in, renf_class& number_field) {
-    throw NumberFieldInputException();
-}
-
-#ifdef ENFNORMALIZ
-template <>
-void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
+void read_number_field_strings(istream& in, string& mp_string, string& indet, string& emb_string) {
     char c;
     string s;
     in >> s;
@@ -588,7 +787,6 @@ void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
         throw BadInputException("Error in reading number field: min_poly does not start with (");
     in >> c;
 
-    string mp_string;
     while (in.good()) {
         c = in.peek();
         if (c == ')') {
@@ -601,24 +799,21 @@ void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
         mp_string += c;
     }
     // omp_set_num_threads(1);
-    
-    string indet;
-    
-    for(auto& g:mp_string){
-        if(isalpha(g)){
+
+    for (auto& g : mp_string) {
+        if (isalpha(g)) {
             indet = g;
             break;
         }
     }
-    
-    if(indet == "e" || indet == "x")
+
+    if (indet == "e" || indet == "x")
         throw BadInputException("Letters e and x not allowed for field generator");
 
     in >> s;
     if (s != "embedding")
         throw BadInputException("Error in reading number field: expected keyword embedding");
     in >> ws;
-    string emb_string;
     c = in.peek();
     if (c == '[') {
         in >> c;
@@ -637,15 +832,23 @@ void read_number_field<renf_elem_class>(istream& in, renf_class& renf) {
 
     if (in.fail())
         throw BadInputException("Could not read number field!");
+}
 
-    renf = renf_class(mp_string, indet, emb_string);
-    renf.gen_name = indet; // temporary fix for bug in renfxx.h
-    
-    renf.set_istream(in); 
+#ifdef ENFNORMALIZ
+renf_class_shared read_number_field(istream& in) {
+    string mp_string, indet, emb_string;
+    read_number_field_strings(in, mp_string, indet, emb_string);
+
+    auto renf = renf_class::make(mp_string, indet, emb_string);
+    // renf->set_pword(in);
+    nmz_set_pword(renf,in);
+
+    return renf;
 }
 #endif
 
-void read_num_param(istream& in, map<NumParam::Param, long>& num_param_input, NumParam::Param numpar, const string& type_string) {
+void read_num_param(istream& in, map<NumParam::Param, long>& num_param_input,
+                    NumParam::Param numpar, const string& type_string) {
     long value;
     in >> value;
     if (in.fail())
@@ -654,11 +857,48 @@ void read_num_param(istream& in, map<NumParam::Param, long>& num_param_input, Nu
 }
 
 template <typename Number>
-map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
-                                                                 OptionsHandler& options,
-                                                                 map<NumParam::Param, long>& num_param_input,
-                                                                 string& polynomial,
-                                                                 renf_class& number_field) {
+void convert_equ_to_inequ(InputMap<Number>& Input, const InputType& equ, const InputType inequ){
+
+    Number MinusOne = -1;
+
+    if(Input.find(equ) != Input.end() && Input[equ].nr_of_rows() > 0){
+        if(Input.find(inequ) == Input.end())
+            Input[inequ] = Matrix<Number>(0, Input[equ][0].size());
+        for(size_t i =0; i< Input[equ].nr_of_rows(); ++i){
+            Input[inequ].append(Input[equ][i]);
+            Input[inequ].append(Input[equ][i]);
+            v_scalar_multiplication<Number>(Input[inequ][Input[inequ].nr_of_rows()-1], MinusOne);
+        }
+        Input[equ].resize(0, Input[equ][0].size());
+    }
+
+}
+
+template <typename Number>
+void convert_equ_to_inequ(InputMap<Number>& Input,  const long dim){
+
+    bool exists_default_breaker = false;
+    for(auto& T: Input){
+        if(is_inequalities(T.first) || is_generators(T.first)){
+            exists_default_breaker = true;
+            break;
+        }
+    }
+
+    convert_equ_to_inequ<Number>(Input, Type::equations, Type::inequalities);
+    convert_equ_to_inequ<Number>(Input, Type::inhom_equations, Type::inhom_inequalities);
+    if(exists_default_breaker)
+        return;
+    Matrix<Number> unit_mat(dim); // must add unit_mat of inequalities to imitate default befavior
+    save_matrix(Input, Type::inequalities, unit_mat); // if no inequalities in input
+}
+
+template <typename Number>
+InputMap<Number> readNormalizInput(istream& in,
+                                            OptionsHandler& options,
+                                            map<NumParam::Param, long>& num_param_input,
+                                            map<PolyParam::Param, vector<string> >& poly_param_input,
+                                            renf_class_shared& number_field) {
     string type_string;
     long i, j;
     long nr_rows, nr_columns, nr_rows_or_columns;
@@ -666,10 +906,15 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
     Number number;
     ConeProperty::Enum cp;
     NumParam::Param numpar;
+    PolyParam::Param polypar;
     set<NumParam::Param> num_par_already_set;
     bool we_have_a_polynomial = false;
+    bool convert_equations = false;
+    size_t length_weight = 0;
+    bool monoid_read = false;
+    bool lattice_ideal_read = false;
 
-    map<Type::InputType, vector<vector<Number> > > input_map;
+    InputMap<Number> input_map;
 
     in >> std::ws;  // eat up any leading white spaces
     int c = in.peek();
@@ -678,8 +923,9 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
     }
     bool new_input_syntax = !std::isdigit(c);
 
+    long dim;
+
     if (new_input_syntax) {
-        long dim;
         while (in.peek() == '/') {
             skip_comment(in);
             in >> std::ws;
@@ -735,6 +981,26 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     num_par_already_set.insert(numpar);
                     continue;
                 }
+               if (isPolyParam(polypar, type_string)) {
+                        if(type_string == "polynomial"){
+                        if (we_have_a_polynomial)
+                                throw BadInputException("Only one polynomial allowed");
+                            we_have_a_polynomial = true;
+                            string poly_str;
+                            read_polynomial(in, poly_str);
+                            poly_param_input[PolyParam::polynomial].push_back(poly_str);
+                        }
+                        else{
+                            vector<string> poly_cosnts;
+                            read_polynomial_constraints(in, poly_cosnts);
+                            poly_param_input[polypar].insert(poly_param_input[polypar].end(),
+                                                             poly_cosnts.begin(), poly_cosnts.end());
+                        }
+                        continue;
+                }
+
+                // One could think that the following are superfluous.
+                // BUT: they take care of having the options in the inpput fike and not ón the command line
                 if (type_string == "LongLong") {
                     options.activateInputFileLongLong();
                     continue;
@@ -743,19 +1009,36 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     options.activateNoExtRaysOutput();
                     continue;
                 }
+                if (type_string == "BinomialsPacked") {
+                    options.activateBinomialsPacked();
+                    continue;
+                }
+                if (type_string == "NoHilbertBasisOutput") {
+                    options.activateNoHilbertBasisOutput();
+                    continue;
+                }
                 if (type_string == "NoMatricesOutput") {
                     options.activateNoMatricesOutput();
                     continue;
                 }
+                if (type_string == "NoOutputOnInterrupt") {
+                    options.activateNoOutputOnInterrupt();
+                    continue;
+                }
                 if (type_string == "NoSuppHypsOutput") {
                     options.activateNoSuppHypsOutput();
                     continue;
                 }
+
                 if (type_string == "number_field") {
 #ifndef ENFNORMALIZ
                     throw BadInputException("number_field only allowed for Normaliz with e-antic");
 #else
-                    read_number_field<Number>(in, number_field);
+                    if (!std::is_same<Number, renf_elem_class>::value) {
+                        throw NumberFieldInputException();
+                    }
+                    // TODO: Err if Number is not renf_elem_class
+                    number_field = read_number_field(in);
 #endif
                     continue;
                 }
@@ -764,8 +1047,9 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         throw BadInputException("Ambient space must be known for " + type_string + "!");
                     }
                     input_type = Type::grading;
-                    save_matrix(input_map, input_type,
-                                vector<vector<Number> >(1, vector<Number>(dim + type_nr_columns_correction(input_type), 1)));
+                    vector<Number> TotDeg(dim + type_nr_columns_correction(input_type),1);
+                    Matrix<Number> TotDegMat(TotDeg);
+                    save_matrix(input_map, input_type, TotDegMat);
                     continue;
                 }
                 if (type_string == "nonnegative") {
@@ -773,8 +1057,8 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         throw BadInputException("Ambient space must be known for " + type_string + "!");
                     }
                     input_type = Type::signs;
-                    save_matrix(input_map, input_type,
-                                vector<vector<Number> >(1, vector<Number>(dim + type_nr_columns_correction(input_type), 1)));
+                    Matrix<Number> SignMat(vector<Number>(dim + type_nr_columns_correction(input_type), 1));
+                    save_matrix(input_map, input_type, SignMat);
                     continue;
                 }
                 if (type_string == "constraints") {
@@ -791,23 +1075,56 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     read_constraints(in, dim, input_map, true);
                     continue;
                 }
-                if (type_string == "polynomial") {
-                    if (we_have_a_polynomial)
-                        throw BadInputException("Only one polynomial allowed");
-                    read_polynomial(in, polynomial);
-                    we_have_a_polynomial = true;
+                if (type_string == "convert_equations") {
+                    convert_equations = true;
+                    continue;
+                }
+                if (type_string == "list_polynomials") {
+                    polynomial_verbose = true;
+                    continue;
+                }
+                if(type_string == "parallel_threads"){
+
+                    long nr_threads;
+                    in >> nr_threads;
+                    if(in.fail())
+                        throw BadInputException("Error after parallel_threads");
+                    set_thread_limit(nr_threads);
                     continue;
                 }
 
                 input_type = to_type(type_string);
+
+                if(type_string == "monoid")
+                    monoid_read = true;
+
+                if(type_string == "lattice_ideal" || type_string == "toric_ideal")
+                    lattice_ideal_read = true;
+
+                if(type_string == "gb_weight"){
+                    if(!(monoid_read || lattice_ideal_read))
+                        throw BadInputException("gb_weight must follow monoid, lattice_ideal or toric_ideal");
+                    if(monoid_read){
+                        length_weight = input_map[Type::monoid].nr_of_rows();
+                    }
+                    else{
+                        length_weight = dim;
+                    }
+                }
+
                 if (dim_known)
                     nr_columns = dim + type_nr_columns_correction(input_type);
 
+                if(type_string == "gb_weight")
+                    nr_columns = length_weight;
+
                 if (type_is_vector(input_type)) {
                     nr_rows_or_columns = nr_rows = 1;
                     in >> std::ws;  // eat up any leading white spaces
                     c = in.peek();
                     if (c == 'u') {  // must be unit vector
+                        if(type_string == "gb_weight")
+                            throw BadInputException("gb_weight cannot be unit vector");
                         string vec_kind;
                         in >> vec_kind;
                         if (vec_kind != "unit_vector") {
@@ -824,7 +1141,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                             throw BadInputException("Ambient space must be known for unit vector " + type_string + "!");
                         }
 
-                        vector<vector<Number> > e_i = vector<vector<Number> >(1, vector<Number>(nr_columns, 0));
+                        Matrix<Number> e_i = Matrix<Number>(vector<Number>(nr_columns, 0));
                         if (pos < 1 || pos > static_cast<long>(e_i[0].size())) {
                             throw BadInputException("Error while reading " + type_string + " as a unit_vector " + toString(pos) +
                                                     "!");
@@ -847,12 +1164,12 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         }
 
                         vector<Number> sparse_vec;
-                        nr_columns = dim + type_nr_columns_correction(input_type);
+                        // nr_columns = dim + type_nr_columns_correction(input_type);
                         bool success = read_sparse_vector(in, sparse_vec, nr_columns);
                         if (!success) {
                             throw BadInputException("Error while reading " + type_string + " as a sparse vector!");
                         }
-                        save_matrix(input_map, input_type, vector<vector<Number> >(1, sparse_vec));
+                        save_matrix(input_map, input_type, Matrix<Number>(sparse_vec));
                         continue;
                     }
 
@@ -867,7 +1184,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         if (!success || (long)formatted_vec.size() != nr_columns) {
                             throw BadInputException("Error while reading " + type_string + " as a formatted vector!");
                         }
-                        save_matrix(input_map, input_type, vector<vector<Number> >(1, formatted_vec));
+                        save_matrix(input_map, input_type, Matrix<Number>(formatted_vec));
                         continue;
                     }  // end formatted vector
                 }
@@ -875,7 +1192,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     in >> std::ws;
                     c = in.peek();
 
-                    if (c != '[' && !std::isdigit(c)) {  // must be transpose
+                    if (c != '[' && c != 'u' && !std::isdigit(c)) {  // must be transpose
                         string transpose_str;
                         in >> transpose_str;
                         if (transpose_str != "transpose") {
@@ -886,12 +1203,12 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         c = in.peek();
                     }
                     if (c == '[') {  // it is a formatted matrix
-                        vector<vector<Number> > formatted_mat;
+                        Matrix<Number> formatted_mat;
                         bool success = read_formatted_matrix(in, formatted_mat, transpose);
                         if (!success) {
                             throw BadInputException("Error while reading formatted matrix " + type_string + "!");
                         }
-                        if (formatted_mat.size() == 0) {  // empty matrix
+                        if (formatted_mat.nr_of_rows() == 0) {  // empty matrix
                             input_type = to_type(type_string);
                             save_empty_matrix(input_map, input_type);
                             continue;
@@ -908,7 +1225,27 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
 
                         save_matrix(input_map, input_type, formatted_mat);
                         continue;
-                    }  // only plain matrix left
+                    }
+                    if (c == 'u') {  // must be unit matrix
+                        string unit_test;
+                        in >> unit_test;
+                        if (unit_test != "unit_matrix") {
+                            throw BadInputException("Error while reading " + type_string + ": unit matrix expected!");
+                        }
+                        if (!dim_known) {
+                            throw BadInputException("Dimension must be known for unit matrix!");
+                        }
+                        nr_columns = dim + type_nr_columns_correction(input_type);
+                        Matrix<Number> unit_mat(nr_columns,nr_columns);
+                        nr_columns = dim + type_nr_columns_correction(input_type);
+                        for (long i = 0; i < nr_columns; ++i) {
+                            unit_mat[i][i] = 1;
+                        }
+                        save_matrix(input_map, input_type, unit_mat);
+                        continue;
+                    }
+
+                    // only plain matrix left
 
                     in >> nr_rows_or_columns;      // is number of columns if transposed
                     nr_rows = nr_rows_or_columns;  // most of the time
@@ -931,10 +1268,12 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     continue;
                 }
 
-                vector<vector<Number> > M(nr_rows);
+                Matrix<Number> M(nr_rows,nr_columns);
+                bool dense_matrix = true;
                 in >> std::ws;
                 c = in.peek();
                 if (c == 's') {  // must be sparse
+                    dense_matrix = false;
                     string sparse_test;
                     in >> sparse_test;
                     if (sparse_test != "sparse") {
@@ -947,7 +1286,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                         }
                     }
                 }
-                else {  // dense matrix
+                if (dense_matrix) {  // dense matrix
                     for (i = 0; i < nr_rows; i++) {
                         M[i].resize(nr_columns);
                         for (j = 0; j < nr_columns; j++) {
@@ -957,7 +1296,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
                     }
                 }
                 if (transpose)
-                    M = transpose_mat(M);
+                    M = M.transpose();
                 save_matrix(input_map, input_type, M);
             }
             if (in.fail()) {
@@ -976,7 +1315,7 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
             if ((nr_rows < 0) || (nr_columns < 0)) {
                 throw BadInputException("Error while reading a " + toString(nr_rows) + "x" + toString(nr_columns) + " matrix !");
             }
-            vector<vector<Number> > M(nr_rows, vector<Number>(nr_columns));
+            Matrix<Number> M(nr_rows, nr_columns);
             for (i = 0; i < nr_rows; i++) {
                 for (j = 0; j < nr_columns; j++) {
                     read_number(in, M[i][j]);
@@ -995,21 +1334,30 @@ map<Type::InputType, vector<vector<Number> > > readNormalizInput(istream& in,
             save_matrix(input_map, input_type, M);
         }
     }
+    if(convert_equations){
+            convert_equ_to_inequ(input_map, dim);
+    }
     return input_map;
 }
 
-template map<Type::InputType, vector<vector<mpq_class> > > readNormalizInput(istream& in,
-                                                                 OptionsHandler& options,
-                                                                 map<NumParam::Param, long>& num_param_input,
-                                                                 string& polynomial,
-                                                                 renf_class& number_field);
+template InputMap<mpq_class> readNormalizInput(istream& in,
+                                                                             OptionsHandler& options,
+                                                                             map<NumParam::Param, long>& num_param_input,
+                                                                             map<PolyParam::Param, vector<string> >& poly_param_input,
+                                                                             renf_class_shared& number_field);
 
 #ifdef ENFNORMALIZ
-template map<Type::InputType, vector<vector<renf_elem_class> > > readNormalizInput(istream& in,
-                                                                 OptionsHandler& options,
-                                                                 map<NumParam::Param, long>& num_param_input,
-                                                                 string& polynomial,
-                                                                 renf_class& number_field);
+template InputMap<renf_elem_class> readNormalizInput(istream& in,
+                                                                                   OptionsHandler& options,
+                                                                                   map<NumParam::Param, long>& num_param_input,
+                                                                                   map<PolyParam::Param, vector<string> >& poly_param_input,
+                                                                                   renf_class_shared& number_field);
 #endif
 
-} // namespace
+#ifndef NMZ_MIC_OFFLOAD  // offload with long is not supported
+template Matrix<long> readMatrix(const string project);
+#endif  // NMZ_MIC_OFFLOAD
+template Matrix<long long> readMatrix(const string project);
+template Matrix<mpz_class> readMatrix(const string project);
+
+}  // namespace libnormaliz
