@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -28,11 +28,13 @@
 #include <vector>
 #include <ostream>
 #include <list>
+#include <string>
 
 #include "libnormaliz/general.h"
 #include "libnormaliz/integer.h"
 // #include "libnormaliz/convert.h"
 #include "libnormaliz/dynamic_bitset.h"
+#include "libnormaliz/my_omp.h"
 
 #ifdef NMZ_FLINT
 #include "flint/flint.h"
@@ -41,6 +43,7 @@
 
 namespace libnormaliz {
 using std::vector;
+using std::string;
 
 //---------------------------------------------------------------------------
 //				Output
@@ -55,6 +58,40 @@ std::ostream& operator<<(std::ostream& out, const vector<T>& vec) {
     return out;
 }
 
+/*
+ *
+ * NOT O:K.
+ *
+template <typename T>
+void do_print_vector(vector<T>& vec, std::ostream out, int level, int max_level){
+    string indent;
+    for(size_t i = 0; i < max_level - level; ++i)
+        indent += "  ";
+    out << indent << "[" << std::endl;
+    if(level == 0){
+        for(site_t k = 0; k < vec.size(); ++k){
+            out << vec[k];
+            if(k < vec.size() -1)
+                out << ",";
+        }
+    }
+    if(level > 0){
+        for(size_t j = 0; j < vec.size(); ++j){
+            do_print_vector(vec[j], level - 1, max_level);
+            out << indent << "]";
+            if(j < vec.size() -1)
+                out << ",";
+            out << std::endl;
+        }
+    }
+
+}
+
+void print_vector(vector<T>& vec, std::ostream out, int lavel){
+    do_print_vector(vec, out, level, level);
+}
+*/
+
 //---------------------------------------------------------------------------
 //          Prototypes for vector_operations.cpp
 //---------------------------------------------------------------------------
@@ -76,6 +113,8 @@ void order_by_perm_bool(vector<bool>& v, const vector<key_t>& permfix);
 template <typename Integer>
 vector<Integer> v_select_coordinates(const vector<Integer>& v, const vector<key_t> projection_key);
 template <typename Integer>
+dynamic_bitset v_support(const vector<Integer>& v);
+template <typename Integer>
 vector<Integer> v_insert_coordinates(const vector<Integer>& v, const vector<key_t> projection_key, const size_t nr_cols);
 
 //---------------------------------------------------------------------------
@@ -86,7 +125,7 @@ vector<Integer> v_insert_coordinates(const vector<Integer>& v, const vector<key_
 // template<typename Integer>
 template <typename Integer>
 Integer v_scalar_product_vectors_unequal_lungth(const vector<Integer>& a, const vector<Integer>& b) {
-    size_t n = min(a.size(), b.size());
+    size_t n = std::min(a.size(), b.size());
     vector<Integer> trunc_a = a;
     vector<Integer> trunc_b = b;
     trunc_a.resize(n);
@@ -136,9 +175,9 @@ void order_by_perm(vector<T>& v, const vector<key_t>& permfix) {
         inv[perm[i]] = i;
     for (key_t i = 0; i < perm.size(); ++i) {
         key_t j = perm[i];
-        swap(v[i], v[perm[i]]);
-        swap(perm[i], perm[inv[i]]);
-        swap(inv[i], inv[j]);
+        std::swap(v[i], v[perm[i]]);
+        std::swap(perm[i], perm[inv[i]]);
+        std::swap(inv[i], inv[j]);
     }
 }
 
@@ -151,7 +190,7 @@ inline vector<key_t> conjugate_perm(const vector<key_t>& p, const vector<key_t>&
 
     vector<int> inv_k(p.size(), -1);
     for (size_t i = 0; i < k.size(); ++i) {
-        inv_k[k[i]] = i;
+        inv_k[k[i]] = static_cast<int>(i);
     }
     vector<key_t> conj(k.size());
     for (size_t i = 0; i < k.size(); ++i) {
@@ -212,7 +251,7 @@ bool v_non_negative(const vector<Integer>& v) {
 
 //---------------------------------------------------------------------------
 /*
-// returns a key vector containing the positions of non-zero entrys of v
+// returns a key vector containing the positions of non-zero entries of v
 template <typename Integer>
 vector<key_t> v_non_zero_pos(const vector<Integer>& v) {
     vector<key_t> key;
@@ -271,7 +310,7 @@ inline mpq_class v_gcd(const vector<mpq_class>& v) {
 #ifdef ENFNORMALIZ
 
 inline mpz_class get_gcd_num(const renf_elem_class& x) {
-    vector<mpz_class> numerator = x.get_num_vector();
+    vector<mpz_class> numerator = x.num_vector();
     return v_gcd(numerator);
 }
 
@@ -307,7 +346,6 @@ Integer v_lcm(const vector<Integer>& v) {
     return g;
 }
 
-
 // returns lcm of the elements of v from index k up to index j
 template <typename Integer>
 Integer v_lcm_to(const vector<Integer>& v, const size_t k, const size_t j) {
@@ -323,7 +361,6 @@ Integer v_lcm_to(const vector<Integer>& v, const size_t k, const size_t j) {
     return g;
 }
 
-
 //---------------------------------------------------------------------------
 
 template <typename Integer>
@@ -546,7 +583,10 @@ inline void vector2fmpq_poly(fmpq_poly_t flp, const std::vector<mpq_class>& poly
 
     fmpq_poly_fit_length(flp, n);
     for (size_t i = 0; i < poly_vector.size(); ++i) {
-        fmpq_poly_set_coeff_mpq(flp, (slong)i, poly_vector[i].get_mpq_t());
+        fmpq_t fcurrent_coeff;
+        fmpq_init(fcurrent_coeff);
+        fmpq_set_mpq(fcurrent_coeff, poly_vector[i].get_mpq_t());
+        fmpq_poly_set_coeff_fmpq(flp, (slong)i, fcurrent_coeff);
     }
 }
 
@@ -559,8 +599,11 @@ inline void fmpq_poly2vector(std::vector<mpq_class>& poly_vector, const fmpq_pol
     poly_vector.resize(length);
     for (slong i = 0; i < length; i++) {
         mpq_t current_coeff;
+        fmpq_t fcurrent_coeff;
         mpq_init(current_coeff);
-        fmpq_poly_get_coeff_mpq(current_coeff, flp, (slong)i);
+        fmpq_init(fcurrent_coeff);
+        fmpq_poly_get_coeff_fmpq(fcurrent_coeff, flp, (slong)i);
+        fmpq_get_mpq(current_coeff, fcurrent_coeff);
         poly_vector[i] = mpq_class(current_coeff);
     }
 }
@@ -569,7 +612,7 @@ template <>
 inline void make_integral(vector<renf_elem_class>& vec) {
     mpz_class denom = 1;
     for (size_t i = 0; i < vec.size(); ++i) {
-        denom = libnormaliz::lcm(denom, vec[i].get_den());
+        denom = libnormaliz::lcm(denom, vec[i].den());
     }
     renf_elem_class fact(denom);
     if (fact != 1)
@@ -767,10 +810,15 @@ inline renf_elem_class v_scalar_product(const vector<renf_elem_class>& av, const
 
     renf_elem_class ans = 0;
     size_t n = av.size();
+    renf_elem_class help;
 
     for (size_t i = 0; i < n; ++i) {
-        if (av[i] != 0 && bv[i] != 0)
+        if (av[i] != 0 && bv[i] != 0) {
             ans += av[i] * bv[i];
+            /* help = av[i];
+            help *= bv[i]; // does not seem to help
+            ans += help;*/
+        }
     }
     return ans;
 }
@@ -782,11 +830,10 @@ inline renf_elem_class v_scalar_product(const vector<renf_elem_class>& av, const
 template <>
 inline mpq_class v_scalar_product(const vector<mpq_class>& av, const vector<mpq_class>& bv) {
     // loop stretching ; brings some small speed improvement
-    
+
     assert(false);
     return 0;
-    
-} 
+}
 
 /* body removed for the time being
     mpq_class ans = 0;
@@ -869,6 +916,19 @@ inline mpq_class v_scalar_product(const vector<mpq_class>& av, const vector<mpq_
 
 //---------------------------------------------------------------------------
 
+template <typename Integer>
+Integer pos_degree(const vector<Integer>& to_test, const vector<Integer> grading){
+    assert(to_test.size() == grading.size());
+    Integer deg = 0;
+    for(size_t j = 0; j < to_test.size(); ++j)
+        if(to_test[j] > 0)
+            deg += to_test[j]*grading[j];
+
+    return deg;
+}
+
+//---------------------------------------------------------------------------
+
 template <typename Integer>
 vector<Integer> v_select_coordinates(const vector<Integer>& v, const vector<key_t> projection_key) {
     vector<Integer> w(projection_key.size());
@@ -877,6 +937,25 @@ vector<Integer> v_select_coordinates(const vector<Integer>& v, const vector<key_
     return w;
 }
 
+//---------------------------------------------------------------------------
+template <typename Integer>
+dynamic_bitset v_support(const vector<Integer>& v){
+    dynamic_bitset supp(v.size());
+    for(size_t i = 0; i < v.size(); i++){
+        if(v[i] != 0)
+            supp[i] = 1;
+    }
+    return supp;
+}
+
+//---------------------------------------------------------------------------
+
+template <typename Integer>
+void v_transfer_coordinates(vector<Integer>& v, const vector<Integer>& w,   const vector<key_t> insertion_key) {
+    for (size_t i = 0; i< insertion_key.size(); ++i)
+        v[insertion_key[i]] = w[i];
+}
+
 //---------------------------------------------------------------------------
 
 template <typename Integer>
@@ -890,8 +969,7 @@ vector<Integer> v_insert_coordinates(const vector<Integer>& v, const vector<key_
 }
 //---------------------------------------------------------------------------
 
-
-inline  nmz_float l1norm(vector<nmz_float>& v) {
+inline nmz_float l1norm(vector<nmz_float>& v) {
     size_t i, size = v.size();
     nmz_float g = 0;
     for (i = 0; i < size; i++) {
@@ -919,7 +997,7 @@ mpq_class l1norm(vector<mpq_class>& v) {
 
 /* for nmz_float is norms the vector to l_1 norm 1.
  *
- * for mpq_class and renf_elem_class it makes the vector coefficents integral
+ * for mpq_class and renf_elem_class it makes the vector coefficients integral
  *
  * then it extracts the gcd of the coefficients
  */
@@ -947,7 +1025,6 @@ Integer v_make_prime(vector<Integer>& v) {
     return g;
 }
 
-
 template <>
 inline nmz_float v_make_prime(vector<nmz_float>& v) {
     size_t i, size = v.size();
@@ -960,11 +1037,10 @@ inline nmz_float v_make_prime(vector<nmz_float>& v) {
     return g;
 }
 
-
 //---------------------------------------------------------------
 
 // swaps entry i and j of the vector<bool> v
-inline  void v_bool_entry_swap(vector<bool>& v, size_t i, size_t j) {
+inline void v_bool_entry_swap(vector<bool>& v, size_t i, size_t j) {
     if (v[i] != v[j]) {
         v[i].flip();
         v[j].flip();
@@ -976,21 +1052,21 @@ inline  void v_bool_entry_swap(vector<bool>& v, size_t i, size_t j) {
 inline vector<key_t> identity_key(size_t n) {
     vector<key_t> key(n);
     for (size_t k = 0; k < n; ++k)
-        key[k] = k;
+        key[k] = static_cast<key_t>(k);
     return key;
 }
 
 inline vector<key_t> reverse_key(size_t n) {
     vector<key_t> key(n);
     for (size_t k = 0; k < n; ++k)
-        key[k] = (n - 1) - k;
+        key[k] = static_cast<key_t>((n - 1) - k);
     return key;
 }
 
 inline vector<key_t> random_key(size_t n) {
     vector<key_t> key = identity_key(n);
-    for (size_t k = 0; k < 3*n; ++k)
-        swap(key[rand() % n], key[rand() % n]);
+    for (size_t k = 0; k < 3 * n; ++k)
+        std::swap(key[rand() % n], key[rand() % n]);
     return key;
 }
 
@@ -1004,8 +1080,8 @@ inline void order_by_perm_bool(vector<bool>& v, const vector<key_t>& permfix) {
         key_t j = perm[i];
         // v.swap(v[i],v[perm[i]]);
         v_bool_entry_swap(v, i, perm[i]);
-        swap(perm[i], perm[inv[i]]);
-        swap(inv[i], inv[j]);
+        std::swap(perm[i], perm[inv[i]]);
+        std::swap(inv[i], inv[j]);
     }
 }
 
@@ -1030,8 +1106,6 @@ inline void v_scalar_division(vector<nmz_float>& v, const nmz_float scalar) {
     }
 }
 
-
-
 template <>
 inline void v_scalar_division(vector<mpq_class>& v, const mpq_class scalar) {
     size_t i, size = v.size();
@@ -1041,7 +1115,6 @@ inline void v_scalar_division(vector<mpq_class>& v, const mpq_class scalar) {
     }
 }
 
-
 #ifdef ENFNORMALIZ
 template <>
 inline void v_scalar_division(vector<renf_elem_class>& v, const renf_elem_class scalar) {
@@ -1155,7 +1228,6 @@ inline renf_elem_class v_standardize(vector<renf_elem_class>& v, const vector<re
 }
 #endif
 
-
 /* Not used presently
 // the following function removes the denominators and then extracts the Gcd of the numerators
 mpq_class v_standardize(vector<mpq_class>& v, const vector<mpq_class>& LF){
@@ -1209,21 +1281,189 @@ inline dynamic_bitset bool_to_bitset(const vector<bool>& val) {
 inline vector<key_t> bitset_to_key(const dynamic_bitset& val) {
     vector<key_t> ret;
     for (size_t i = 0; i < val.size(); ++i)
-        if(val[i])
-            ret.push_back(i);
+        if (val[i])
+            ret.push_back(static_cast<key_t>(i));
     return ret;
 }
 
-inline dynamic_bitset key_to_bitset(const vector<key_t>& key, long size){
-    
+inline dynamic_bitset key_to_bitset(const vector<key_t>& key, long size) {
     dynamic_bitset bs(size);
-    for(size_t i=0; i< key.size(); ++i){
+    for (size_t i = 0; i < key.size(); ++i) {
         assert(key[i] < size);
-        bs[key[i]] = 1;        
+        bs[key[i]] = 1;
     }
-    return bs;    
+    return bs;
 }
 
+template <typename T>
+vector<bool> binary_expansion(T n) {
+    vector<bool> bin;
+    while (n != 0) {
+        bin.push_back(n & 1);
+        n = n >> 1;
+    }
+    return bin;
+}
+
+template <typename Integer>
+Integer vector_sum_cascade(vector<Integer>& summands) {
+    size_t step = 2;
+    bool added = true;
+    while (added) {
+        added = false;
+#pragma omp parallel for
+        for (size_t k = 0; k < summands.size(); k += step) {
+            if (summands.size() > k + step / 2) {
+                summands[k] += summands[k + step / 2];
+                added = true;
+            }
+        }
+        step *= 2;
+    }
+    return summands[0];
+}
+
+//--------------------------------------------------------------
+
+template <typename Integer>
+class AdditionPyramid {
+   public:
+    vector<Integer> accumulator;
+    vector<size_t> counter;
+    size_t capacity;
+    void add_inner(const Integer summand, const size_t level);
+
+    AdditionPyramid();
+    AdditionPyramid(const size_t& given_capacity);
+    void add(const Integer& summand);
+    Integer sum();
+    void reset();
+    void set_capacity(const size_t& given_capacity);
+};
+
+template <typename Integer>
+void AdditionPyramid<Integer>::add_inner(const Integer summand, const size_t level) {
+    // cout << "***** " << summand << " -- " << level << endl;
+
+    assert(level <= counter.size());
+
+    if (level == counter.size()) {
+        counter.resize(level + 1);
+        accumulator.resize(level + 1);
+        // cout << "$$$$$ " << accumulator[level] << " -- " << summand << endl;
+        accumulator[level] = summand;
+        // cout << "+++ " << accumulator[level] << endl;
+        return;
+    }
+
+    counter[level]++;
+
+    if (counter[level] < capacity) {
+        accumulator[level] += summand;
+        return;
+    }
+
+    add_inner(accumulator[level], level + 1);
+    counter[level] = 0;
+    accumulator[level] = summand;
+}
+
+template <typename Integer>
+AdditionPyramid<Integer>::AdditionPyramid() {
+}
+
+template <typename Integer>
+void AdditionPyramid<Integer>::reset() {
+    counter.clear();
+    accumulator.clear();
+}
+
+template <typename Integer>
+AdditionPyramid<Integer>::AdditionPyramid(const size_t& given_capacity) {
+    capacity = given_capacity;
+    reset();
+}
+
+template <typename Integer>
+void AdditionPyramid<Integer>::set_capacity(const size_t& given_capacity) {
+    capacity = given_capacity;
+}
+
+template <typename Integer>
+Integer AdditionPyramid<Integer>::sum() {
+    Integer our_sum;  // this version works also for CoCoALib::Bigrat
+    our_sum = 0;
+    for (size_t i = 0; i < accumulator.size(); ++i)
+        our_sum += accumulator[i];
+    return our_sum;
+}
+
+template <typename Integer>
+void AdditionPyramid<Integer>::add(const Integer& summand) {
+    if (counter.size() > 0) {
+        if (counter[0] < capacity - 1) {
+            counter[0]++;
+            accumulator[0] += summand;
+            return;
+        }
+    }
+    add_inner(summand, 0);
+}
+
+template <typename T>
+void v_cyclic_shift_right( T& vec, size_t col){
+    if(vec.size() == 0)
+        return;
+    assert(col < vec.size());
+    auto dummy = vec[col];
+    for(int i = col; i >= 1; --i)
+        vec[i] = vec[i-1];
+    vec[0] = dummy;
+}
+
+template <typename T>
+void v_cyclic_shift_left( T& vec, size_t col){
+    if(vec.size() == 0)
+        return;
+    assert(col < vec.size());
+    auto dummy = vec[0];
+    for(size_t i = 0; i < col; ++i)
+        vec[i] = vec[i+1];
+    vec[col] = dummy;
+}
+
+template <typename T>
+T v_permute_coordinates(const T& vec, const vector<key_t>& perm){
+    assert(vec.size() == perm.size());
+    T new_vec(vec.size());
+    for(size_t i = 0; i< vec.size(); ++i)
+        new_vec[i] = vec[perm[i]];
+    return new_vec;
+}
+
+template <typename T>
+T v_inverse_permute_coordinates(const T& vec, const vector<key_t>& perm){
+    assert(vec.size() == perm.size());
+    T new_vec(vec.size());
+    for(size_t i = 0; i< vec.size(); ++i)
+        new_vec[perm[i]] = vec[i];
+    return new_vec;
+}
+
+template <typename T>
+string v_to_point_list(const T& v){
+    string pl;
+    for(auto& c: v)
+        pl += std::to_string(c) + ".";
+    return pl;
+}
+
+inline vector<string> to_string_vector(int argc, char* argv[]){
+    vector<string> ret;
+    for(int i = 0; i < argc; ++i)
+        ret.push_back(string(argv[i]));
+     return ret;
+}
 
 
 }  // namespace libnormaliz
